@@ -1,43 +1,41 @@
-import django
 import pytest
-from crispy_forms import __version__
-from crispy_forms.bootstrap import Field, InlineCheckboxes, UneditableField
+from crispy_forms.bootstrap import (
+    AppendedText,
+    Field,
+    FieldWithButtons,
+    InlineCheckboxes,
+    PrependedAppendedText,
+    PrependedText,
+    StrictButton,
+)
 from crispy_forms.helper import FormHelper
-from crispy_forms.layout import HTML, Column, Fieldset, Layout, Row, Submit
+from crispy_forms.layout import HTML, Column, Fieldset, Layout, Row
 from crispy_forms.utils import render_crispy_form
 from django import forms
 from django.forms.models import formset_factory, modelformset_factory
 from django.middleware.csrf import _get_new_csrf_string
 from django.shortcuts import render
 from django.template import Context, Template
-from django.test.html import parse_html
-from django.test.utils import override_settings
 from django.urls import reverse
 from django.utils.translation import gettext_lazy as _
 
 from .forms import (
-    AdvancedFileForm,
     CheckboxesSampleForm,
     CrispyEmptyChoiceTestModel,
     CrispyTestModel,
     FileForm,
+    FileFormRequired,
+    HelpTextForm,
+    InputsForm,
+    LabelForm,
     SampleForm,
     SampleForm2,
     SampleForm3,
     SampleForm4,
-    SampleForm5,
     SampleForm6,
-    SelectSampleForm,
 )
-from .test_settings import TEMPLATE_DIRS
 from .utils import contains_partial, parse_expected, parse_form
 
-CONVERTERS = {
-    "textinput": "textinput textInput",
-    "fileinput": "fileinput fileUpload",
-    "passwordinput": "textinput textInput",
-}
-
 
 def test_invalid_unicode_characters(settings):
     # Adds a BooleanField that uses non valid unicode characters "ñ"
@@ -202,7 +200,9 @@ def test_layout_fieldset_row_html_with_unicode_fieldnames():
     assert 'id="row_passwords"' in html
     assert html.count("<label") == 6
 
-    assert 'class="form-row rows"' in html
+    assert 'class="row rows"' in html
+    assert 'class="form-label' in html
+
     assert "Hello!" in html
     assert "testLink" in html
 
@@ -271,10 +271,12 @@ def test_column_has_css_classes():
 
     c = Context({"form": form, "form_helper": form_helper})
     html = template.render(c)
+
+    assert html.count("formColumn") == 0
     assert html.count("col-md") == 1
 
 
-def test_bs4_column_css_classes():
+def test_bs5_column_css_classes():
     template = Template(
         """
         {% load crispy_forms_tags %}
@@ -299,6 +301,36 @@ def test_bs4_column_css_classes():
     assert html.count("col-sm") == 1
 
 
+def test_bs5_field_with_buttons_css_classes():
+    form = SampleForm()
+    form.helper = FormHelper()
+    form.helper.add_layout(
+        Layout(
+            Column(
+                FieldWithButtons(
+                    "email",
+                    HTML("""<a role='button' class='btn btn-primary'>click me</a>"""),
+                )
+            ),
+        )
+    )
+
+    assert parse_form(form) == parse_expected("field_with_buttons.html")
+
+    form = SampleForm3({})
+    form.helper = FormHelper()
+    form.helper.add_layout(
+        Layout(
+            Column(
+                FieldWithButtons(
+                    "email", StrictButton("Go!", css_class="btn-outline-secondary")
+                )
+            ),
+        )
+    )
+    assert parse_form(form) == parse_expected("field_with_buttons_failing.html")
+
+
 def test_formset_layout():
     SampleFormSet = formset_factory(SampleForm, extra=3)
     formset = SampleFormSet()
@@ -325,23 +357,23 @@ def test_formset_layout():
     # Check formset fields
     assert contains_partial(
         html,
-        '<input id="id_form-TOTAL_FORMS" name="form-TOTAL_FORMS" '
-        'type="hidden" value="3"/>',
+        '<input id="id_form-TOTAL_FORMS" name="form-TOTAL_FORMS" type="hidden"'
+        ' value="3"/>',
     )
     assert contains_partial(
         html,
-        '<input id="id_form-INITIAL_FORMS" name="form-INITIAL_FORMS" '
-        'type="hidden" value="0"/>',
+        '<input id="id_form-INITIAL_FORMS" name="form-INITIAL_FORMS" type="hidden"'
+        ' value="0"/>',
     )
     assert contains_partial(
         html,
-        '<input id="id_form-MAX_NUM_FORMS" name="form-MAX_NUM_FORMS" '
-        'type="hidden" value="1000"/>',
+        '<input id="id_form-MAX_NUM_FORMS" name="form-MAX_NUM_FORMS" type="hidden"'
+        ' value="1000"/>',
     )
     assert contains_partial(
         html,
-        '<input id="id_form-MIN_NUM_FORMS" name="form-MIN_NUM_FORMS" '
-        'type="hidden" value="0"/>',
+        '<input id="id_form-MIN_NUM_FORMS" name="form-MIN_NUM_FORMS" type="hidden"'
+        ' value="0"/>',
     )
     assert html.count("hidden") == 5
 
@@ -360,7 +392,7 @@ def test_formset_layout():
     assert html.count("Note for first form only") == 1
     assert html.count("row") == 3
 
-    assert html.count("form-group") == 18
+    assert html.count("mb-3") == 21
 
 
 def test_modelformset_layout():
@@ -379,18 +411,18 @@ def test_modelformset_layout():
 
     assert contains_partial(
         html,
-        '<input id="id_form-TOTAL_FORMS" name="form-TOTAL_FORMS" '
-        'type="hidden" value="3"/>',
+        '<input id="id_form-TOTAL_FORMS" name="form-TOTAL_FORMS" type="hidden"'
+        ' value="3"/>',
     )
     assert contains_partial(
         html,
-        '<input id="id_form-INITIAL_FORMS" name="form-INITIAL_FORMS" '
-        'type="hidden" value="0"/>',
+        '<input id="id_form-INITIAL_FORMS" name="form-INITIAL_FORMS" type="hidden"'
+        ' value="0"/>',
     )
     assert contains_partial(
         html,
-        '<input id="id_form-MAX_NUM_FORMS" name="form-MAX_NUM_FORMS" '
-        'type="hidden" value="1000"/>',
+        '<input id="id_form-MAX_NUM_FORMS" name="form-MAX_NUM_FORMS" type="hidden"'
+        ' value="1000"/>',
     )
 
     assert html.count('name="form-0-email"') == 1
@@ -472,20 +504,6 @@ def test_choice_with_none_is_selected():
     assert "checked" in html
 
 
-@override_settings(
-    TEMPLATES=[
-        {
-            "BACKEND": "django.template.backends.django.DjangoTemplates",
-            "DIRS": TEMPLATE_DIRS,
-            "OPTIONS": {
-                "loaders": [
-                    "django.template.loaders.filesystem.Loader",
-                    "django.template.loaders.app_directories.Loader",
-                ],
-            },
-        }
-    ]
-)
 def test_keepcontext_context_manager():
     # Test case for issue #180
     # Apparently it only manifest when using render_to_response this exact way
@@ -500,126 +518,37 @@ def test_keepcontext_context_manager():
     response = render(
         request=None, template_name="crispy_render_template.html", context=context
     )
-
-    assert response.content.count(b"custom-control-inline") == 3
-    assert response.content.count(b"custom-checkbox") > 0
+    print(response.content)
+    assert response.content.count(b"form-check-inline") == 3
+    assert response.content.count(b"form-check-input") > 0
 
 
-def test_use_custom_control_is_used_in_checkboxes():
-    form = CheckboxesSampleForm()
-    form.helper = FormHelper()
-    form.helper.layout = Layout(
-        "checkboxes",
-        InlineCheckboxes("alphacheckboxes"),
-        "numeric_multiple_checkboxes",
-    )
-    # form.helper.use_custom_control take default value which is True
-    assert parse_form(form) == parse_expected(
-        "bootstrap4/test_layout/test_use_custom_control_is_used_in_checkboxes_true.html"
-    )
-
-    form.helper.use_custom_control = True
-    assert parse_form(form) == parse_expected(
-        "bootstrap4/test_layout/test_use_custom_control_is_used_in_checkboxes_true.html"
-    )
-
-    form.helper.use_custom_control = False
-    assert parse_form(form) == parse_expected(
-        "bootstrap4/test_layout/"
-        "test_use_custom_control_is_used_in_checkboxes_false.html"
-    )
-
-    form = CheckboxesSampleForm({})
-    form.helper = FormHelper()
-    form.helper.layout = Layout(
-        "checkboxes",
-        InlineCheckboxes("alphacheckboxes"),
-        "numeric_multiple_checkboxes",
-    )
-    if django.VERSION < (5, 0):
-        expected = (
-            "bootstrap4/test_layout/"
-            "test_use_custom_control_is_used_in_checkboxes_true_failing_lt50.html"
-        )
-    else:
-        expected = (
-            "bootstrap4/test_layout/"
-            "test_use_custom_control_is_used_in_checkboxes_true_failing.html"
-        )
-    assert parse_form(form) == parse_expected(expected)
-
-
-def test_use_custom_control_is_used_in_radio():
-    form = SampleForm5()
-    form.helper = FormHelper()
-    form.helper.layout = Layout(
-        "radio_select",
-    )
-    # form.helper.use_custom_control take default value which is True
-    assert parse_form(form) == parse_expected(
-        "bootstrap4/test_layout/test_use_custom_control_is_used_in_radio_true.html"
-    )
-
-    form.helper.use_custom_control = True
-    assert parse_form(form) == parse_expected(
-        "bootstrap4/test_layout/test_use_custom_control_is_used_in_radio_true.html"
-    )
-
-    form.helper.use_custom_control = False
-    assert parse_form(form) == parse_expected(
-        "bootstrap4/test_layout/test_use_custom_control_is_used_in_radio_false.html"
-    )
-
-    form = SampleForm5({})
-    form.helper = FormHelper()
-    form.helper.layout = Layout(
-        "radio_select",
-    )
-    if django.VERSION < (5, 0):
-        expected = (
-            "bootstrap4/test_layout/"
-            "test_use_custom_control_is_used_in_radio_true_failing_lt50.html"
-        )
-    else:
-        expected = (
-            "bootstrap4/test_layout/"
-            "test_use_custom_control_is_used_in_radio_true_failing.html"
-        )
-    assert parse_form(form) == parse_expected(expected)
-
-
-@pytest.mark.parametrize(
-    "use_custom_control, expected_html",
-    [
-        (True, "bootstrap4/test_layout/test_use_custom_control_in_select_true.html"),
-        (False, "bootstrap4/test_layout/test_use_custom_control_in_select_false.html"),
-    ],
-)
-def test_use_custom_control_in_select(use_custom_control, expected_html):
-    form = SelectSampleForm()
-
-    form.helper = FormHelper()
-    form.helper.template_pack = "bootstrap4"
-    form.helper.layout = Layout("select")
-    form.helper.use_custom_control = use_custom_control
-
-    assert parse_form(form) == parse_expected(expected_html)
-
-
-def test_bootstrap4_form_inline():
+def test_bootstrap5_form_inline():
     form = SampleForm()
     form.helper = FormHelper()
     form.helper.form_class = "form-inline"
-    form.helper.field_template = "bootstrap4/layout/inline_field.html"
+    form.helper.field_template = "bootstrap5/layout/inline_field.html"
     form.helper.layout = Layout("email", "password1", "last_name")
+    form.helper.form_class = "row row-cols-lg-auto"
+    assert parse_form(form) == parse_expected("test_bootstrap5_form_inline.html")
 
-    html = render_crispy_form(form)
-    assert html.count('class="form-inline"') == 1
-    assert html.count('class="input-group"') == 3
-    assert html.count('<label for="id_email" class="sr-only') == 1
-    assert html.count('id="div_id_email" class="input-group"') == 1
-    assert html.count('placeholder="email"') == 1
-    assert html.count("</label> <input") == 3
+
+def test_select():
+    form = InputsForm()
+    form.helper = FormHelper()
+    form.helper.layout = Layout("select_input")
+    assert parse_form(form) == parse_expected("test_select.html")
+
+
+def test_select_prepended():
+    form = InputsForm()
+    form.helper = FormHelper()
+    form.helper.layout = Layout(
+        PrependedText("select_input", "bar"),
+        AppendedText("select_input", "bar"),
+        PrependedAppendedText("select_input", "£", ".00"),
+    )
+    assert parse_form(form) == parse_expected("test_prepended_appended_select.html")
 
 
 def test_update_attributes_class():
@@ -628,7 +557,7 @@ def test_update_attributes_class():
     form.helper.layout = Layout("email", Field("password1"), "password2")
     form.helper["password1"].update_attributes(css_class="hello")
     html = render_crispy_form(form)
-    assert html.count(' class="hello') == 1
+    assert html.count(' class="hello textinput') == 1
     form.helper = FormHelper()
     form.helper.layout = Layout(
         "email",
@@ -637,203 +566,85 @@ def test_update_attributes_class():
     )
     form.helper["password1"].update_attributes(css_class="hello2")
     html = render_crispy_form(form)
-    assert html.count(' class="hello hello2') == 1
+    assert html.count(' class="hello hello2 textinput') == 1
 
 
-@override_settings(CRISPY_CLASS_CONVERTERS=CONVERTERS)
 def test_file_field():
     form = FileForm()
     form.helper = FormHelper()
-    form.helper.field_class = "col-lg-9 mb-2"
     form.helper.layout = Layout("clearable_file")
-    assert parse_form(form) == parse_expected(
-        "bootstrap4/test_layout/test_file_field_clearable_custom_control.html"
-    )
 
-    form.helper.use_custom_control = False
-    assert parse_form(form) == parse_expected(
-        "bootstrap4/test_layout/test_file_field_clearable.html"
-    )
-
-    form.helper.use_custom_control = True
-    form.helper.layout = Layout("file_field")
-    assert parse_form(form) == parse_expected(
-        "bootstrap4/test_layout/test_file_field_custom_control.html"
-    )
-
-    form.helper.use_custom_control = False
-    assert parse_form(form) == parse_expected(
-        "bootstrap4/test_layout/test_file_field_default.html"
-    )
-
-
-def test_file_field_with_custom_class():
-    form = AdvancedFileForm()
-    form.helper = FormHelper()
-    form.helper.layout = Layout("clearable_file")
-    assert parse_form(form) == parse_expected(
-        "bootstrap4/test_layout/test_file_field_with_custom_class_clearable.html"
-    )
+    assert parse_form(form) == parse_expected("test_clearable_file_field.html")
 
     form.helper.layout = Layout("file_field")
-    assert parse_form(form) == parse_expected(
-        "bootstrap4/test_layout/test_file_field_with_custom_class.html"
-    )
 
+    assert parse_form(form) == parse_expected("test_file_field.html")
 
-def test_form_control_size():
-    "CSS classes form-control and form-control-lg are both required"
-    form = SampleForm()
+    form = FileFormRequired({})
     form.helper = FormHelper()
-    form.helper.layout = Layout(Field("first_name", css_class="form-control-lg"))
-    assert parse_form(form) == parse_expected(
-        "bootstrap4/test_layout/test_form_control_size.html"
-    )
+    form.helper.layout = Layout("clearable_file")
+
+    assert parse_form(form) == parse_expected("test_clearable_file_field_failing.html")
+
+    form.helper.layout = Layout("file_field")
+
+    assert parse_form(form) == parse_expected("test_file_field_failing.html")
 
 
-@pytest.mark.skipif(
-    __version__[0] == "1",
-    reason="#1224 changed editable field behaviour post crispy forms 1.x",
-)
-def test_uneditable_field():
+def test_row():
     form = SampleForm()
     form.helper = FormHelper()
     form.helper.layout = Layout(
-        UneditableField("first_name"),
-    )
-    assert parse_form(form) == parse_expected(
-        "bootstrap4/test_layout/test_uneditable_field.html"
+        Row(
+            Column("first_name"),
+            Column("last_name"),
+        )
     )
+    assert parse_form(form) == parse_expected("row.html")
 
 
-@pytest.mark.parametrize(
-    "use_custom_control, expected_html",
-    [
-        (
-            True,
-            "bootstrap4/test_layout/"
-            "test_use_custom_control_in_uneditable_select_true.html",
-        ),
-        (
-            False,
-            "bootstrap4/test_layout/"
-            "test_use_custom_control_in_uneditable_select_false.html",
-        ),
-    ],
-)
-@pytest.mark.skipif(
-    __version__[0] == "1",
-    reason="#1224 changed editable field behaviour post crispy forms 1.x",
-)
-def test_use_custom_control_in_uneditable_select(use_custom_control, expected_html):
-    form = SelectSampleForm()
+def test_html_label_escape():
+    form = LabelForm()
     form.helper = FormHelper()
-    form.helper.template_pack = "bootstrap4"
-    form.helper.layout = Layout(UneditableField("select"))
-    form.helper.use_custom_control = use_custom_control
-    assert parse_form(form) == parse_expected(expected_html)
+    form.helper.layout = Layout("text_input")
+    html = render_crispy_form(form)
+    assert "&lt;&gt;&amp;" in html
 
 
-def test_multiple_fields(settings):
-    "Field can accept any number of fields and apply the kwargs to all fields"
-    form = SampleForm()
-    form.helper = FormHelper()
-    form.helper.layout = Layout(
-        Field("first_name", "last_name", css_class="form-control-lg")
-    )
-    template_pack = settings.CRISPY_TEMPLATE_PACK
-    assert parse_form(form) == parse_expected(
-        f"{template_pack}/test_layout/test_multiple_fields.html"
-    )
-
-
-def test_fundamentals():
-    """
-    This is the example that is in the `crispy_tag_forms` docs.
-    """
-
-    class ExampleForm(forms.Form):
-        like_website = forms.TypedChoiceField(
-            label="Do you like this website?",
-            choices=((1, "Yes"), (0, "No")),
-            coerce=lambda x: bool(int(x)),
-            widget=forms.RadioSelect,
-            initial="1",
-            required=True,
-        )
-
-        favorite_food = forms.CharField(
-            label="What is your favorite food?",
-            max_length=80,
-            required=True,
-        )
-
-        favorite_color = forms.CharField(
-            label="What is your favorite color?",
-            max_length=80,
-            required=True,
-        )
-
-        favorite_number = forms.IntegerField(
-            label="Favorite number",
-            required=False,
-        )
-
-        notes = forms.CharField(
-            label="Additional notes or feedback",
-            required=False,
-        )
-
-        def __init__(self, *args, **kwargs):
-            super().__init__(*args, **kwargs)
-            self.helper = FormHelper()
-            self.helper.form_id = "id-exampleForm"
-            self.helper.form_class = "blueForms"
-            self.helper.form_method = "post"
-            self.helper.form_action = "submit_survey"
-
-            self.helper.add_input(Submit("submit", "Submit"))
-
-    form = ExampleForm()
-    context = {"csrf_token": "NotARealToken"}
-    html = render_crispy_form(form, context=context)
-    assert parse_html(html) == parse_expected(
-        "bootstrap4/test_layout/test_fundamentals_example.html"
-    )
-
-
-@pytest.mark.skipif(
-    __version__[0] == "1", reason="#1250 removed custom-control post crispy forms 1.x"
-)
-def test_table_inline_formset_checkbox():
-    class TestForm(forms.Form):
-        box_one = forms.CharField(label="box one", widget=forms.CheckboxInput())
-        box_two = forms.CharField(label="box two", widget=forms.CheckboxInput())
-
-    formset = formset_factory(TestForm)
+def test_tabular_formset_layout():
+    SampleFormSet = formset_factory(SampleForm, extra=3)
+    formset = SampleFormSet()
     formset.helper = FormHelper()
-    formset.helper.template = "bootstrap4/table_inline_formset.html"
+    formset.helper.template = "bootstrap5/table_inline_formset.html"
+    assert parse_form(formset) == parse_expected("test_tabular_formset_layout.html")
+
+    SampleFormSet = formset_factory(SampleForm, extra=3)
+    data = {
+        "form-TOTAL_FORMS": "1",
+        "form-INITIAL_FORMS": "0",
+    }
+    formset = SampleFormSet(data)
+    formset.helper = FormHelper()
+    formset.helper.template = "bootstrap5/table_inline_formset.html"
     assert parse_form(formset) == parse_expected(
-        "bootstrap4/test_layout/test_inline_formset_checkbox.html"
+        "test_tabular_formset_layout_failing.html"
     )
 
 
-def test_radio_attrs():
-    class TestForm(forms.Form):
-        radios = forms.ChoiceField(
-            choices=(
-                ("option_one", "Option one"),
-                ("option_two", "Option two"),
-            ),
-            widget=forms.RadioSelect(
-                attrs={
-                    "class": "sr-only sr-only-focusable",
-                }
-            ),
-        )
-
-    form = TestForm()
+def test_flat_attrs_safe():
+    form = SampleForm()
     form.helper = FormHelper()
-    assert parse_form(form) == parse_expected(
-        "bootstrap4/test_layout/test_radio_attrs.html"
+    form.helper.layout = Layout(
+        Row(
+            aria_labelledby="test<>%",
+        )
     )
+    form.helper.form_tag = False
+    assert parse_form(form) == parse_expected("flat_attrs.html")
+
+
+def test_help_text_no_escape():
+    form = HelpTextForm()
+    form.helper = FormHelper()
+    form.helper.form_tag = False
+    assert parse_form(form) == parse_expected("help_text_escape.html")
